@@ -38,10 +38,17 @@ class RestAPI(object):
 		Define your additional user data in the __default_user_data__ variable.
 		"""
 		self.__default_user_data__ = {
+			# the users api key.
 			"api_key":None,
+			# the subscription plan.
 			"membership":"free",
+			# the requests variables.
 			"requests":0,
 			"timestamp":None,
+			# the signed up timestamp.
+			"signed_up":None,
+			# the email permission,
+			"email_permission":True,
 		}
 
 		"""
@@ -414,6 +421,23 @@ class RestAPI(object):
 		return r3sponse.success_response("Successfully synchronized the users.", log_level=log_level)
 
 		#
+	def set_email_permission(self uid, permission=True):
+		
+		# load user data.
+		reference = f"users/{uid}"
+		response = self.firebase.firestore.load(reference) 
+		if not r3sponse.success(response): return response
+		data = response["data"]
+
+		# edit data.
+		data["email_permission"] = permission
+
+		# save data.
+		response = self.firebase.firestore.save(reference, data) 
+		if response["error"] != None: return response
+
+		# handlers.
+		return r3sponse.success_response(f"Successfully set the email permission of user [{uid}] to [{permission}].")
 
 	# requests & rate limit functions.
 	def verify_rate_limit(self, 
@@ -525,6 +549,9 @@ class RestAPI(object):
 		reference = f"users/{uid}"
 		data = ast.literal_eval(str(self.__default_user_data__))
 
+		# set signed up timestamp.
+		data["signed_up"] = Formats.Date().date
+
 		# generate api key.
 		response = self.__generate_key__()
 		if response["error"] != None: return response
@@ -552,6 +579,12 @@ class RestAPI(object):
 		if data != clone:
 			edits += 1
 
+		# test signed up stamp.
+		try: data["signed_up"]
+		except KeyError: 
+			data["signed_up"] = Formats.Date().date
+			edits += 1
+
 		# test api key.
 		try: data["api_key"]
 		except KeyError: 
